use iced::task::{Never, Sipper, sipper};
use tokio::time::sleep;
use std::time::Duration;

use nusb::{transfer::{ RequestBuffer, TransferError }, Interface};

static USB_VENDOR: u16  = 0xffff;
static USB_PRODUCT: u16 = 0x0035;


fn get_interface() -> Option<Interface> {
    let dev = nusb::list_devices().ok()?.find(|dev| dev.vendor_id() == USB_VENDOR && dev.product_id() == USB_PRODUCT)?;
    let device = dev.open().ok()?;
    device.detach_and_claim_interface(0).ok()
}


pub fn connect() -> impl Sipper<Never, Event> {
    sipper(async |mut output|{
        loop {
            let interface = loop {
                if let Some(interface) = get_interface(){
                    break interface;
                }
                sleep(Duration::new(0, 100)).await;
            };
            output.send(Event::Connected).await;
            let mut card_id = 0;
            loop {
                let buf = RequestBuffer::new(8);
                let data = interface.interrupt_in(0x81, buf).await;
                match data.status {
                    Err(TransferError::Disconnected) => {
                        output.send(Event::Disconnected).await;
                        break;
                    },
                    Ok(()) => {
                        let code = data.data[2];
                        match code {
                            40 => {
                                output.send(Event::ReadedCardId(card_id)).await;
                                card_id = 0;
                            },
                            0 => (),
                            x @ 30..=38 => {
                                card_id *= 10;
                                card_id += x as usize - 29;
                            },
                            39 => {
                                card_id *= 10;
                            },
                            _ => panic!(),
                        }
                    },
                    _ => todo!(),
                }
            }
        }
    })
}

pub enum Event{
    Disconnected,
    Connected,
    ReadedCardId(usize),
}
