use iced::futures;
use iced::task::{Never, Sipper, sipper};

use futures::stream::StreamExt;

use async_tungstenite::tungstenite;
use crate::config::CONFIG;

pub fn connect() -> impl Sipper<Never, Event> {
    sipper(async |mut output| {
        let endpoint_addr: String = CONFIG.WSS_HOST.clone() + "/api/live/racers";
        loop {

            let mut websocket =
                match async_tungstenite::tokio::connect_async(endpoint_addr.as_str()).await
                {
                    Ok((websocket, _)) => {
                        websocket.fuse()
                    }
                    Err(_) => {
                        tokio::time::sleep(tokio::time::Duration::from_secs(1))
                            .await;
                        continue;
                    }
                };
            println!("Websocket Connected!");
            loop {
                futures::select! {
                    received = websocket.select_next_some() => {
                        match received {
                            Ok(tungstenite::Message::Text(message)) => {
                                output.send(Event::MessageReceived(String::from(message.as_str()))).await;
                            }
                            Err(_) => {
                                println!("Websocket Disconnected!");
                                break;
                            }
                            Ok(_) => {},
                        }
                    }
                }
            }
        }
    })
}

#[derive(Debug, Clone)]
pub enum Event {
    MessageReceived(String),
}
